@@ -25,17 +25,13 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
-from __future__ import absolute_import, unicode_literals
+from __future__ import annotations
 
 from .utils import bytes2int, int2bytes
 from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives import hashes, serialization
-from cryptography.hazmat.primitives.asymmetric import ec, rsa, padding
-
-try:
-    from cryptography.hazmat.primitives.asymmetric import ed25519
-except ImportError:  # EdDSA requires Cryptography >= 2.6.
-    ed25519 = None
+from cryptography.hazmat.primitives.asymmetric import ec, rsa, padding, ed25519
+from typing import Sequence, Type, Mapping, Any, Union, TypeVar
 
 
 class CoseKey(dict):
@@ -45,9 +41,9 @@ class CoseKey(dict):
     :cvar ALGORITHM: COSE algorithm identifier.
     """
 
-    ALGORITHM = None
+    ALGORITHM: int = None  # type: ignore
 
-    def verify(self, message, signature):
+    def verify(self, message: bytes, signature: bytes) -> None:
         """Validates a digital signature over a given message.
 
         :param message: The message which was signed.
@@ -56,7 +52,10 @@ class CoseKey(dict):
         raise NotImplementedError("Signature verification not supported.")
 
     @classmethod
-    def from_cryptography_key(cls, public_key):
+    def from_cryptography_key(
+        cls: Type[T_CoseKey],
+        public_key: Union[rsa.RSAPublicKey, ec.EllipticCurvePublicKey],
+    ) -> T_CoseKey:
         """Converts a PublicKey object from Cryptography into a COSE key.
 
         :param public_key: Either an EC or RSA public key.
@@ -65,22 +64,19 @@ class CoseKey(dict):
         raise NotImplementedError("Creation from cryptography not supported.")
 
     @staticmethod
-    def for_alg(alg):
+    def for_alg(alg: int) -> Type[CoseKey]:
         """Get a subclass of CoseKey corresponding to an algorithm identifier.
 
         :param alg: The COSE identifier of the algorithm.
         :return: A CoseKey.
         """
-        if alg == EdDSA.ALGORITHM and ed25519 is None:
-            # EdDSA requires Cryptography >= 2.6.
-            return UnsupportedKey
         for cls in CoseKey.__subclasses__():
             if cls.ALGORITHM == alg:
                 return cls
         return UnsupportedKey
 
     @staticmethod
-    def for_name(name):
+    def for_name(name: str) -> Type[CoseKey]:
         """Get a subclass of CoseKey corresponding to an algorithm identifier.
 
         :param alg: The COSE identifier of the algorithm.
@@ -92,7 +88,7 @@ class CoseKey(dict):
         return UnsupportedKey
 
     @staticmethod
-    def parse(cose):
+    def parse(cose: Mapping[int, Any]) -> CoseKey:
         """Create a CoseKey from a dict"""
         alg = cose.get(3)
         if not alg:
@@ -100,15 +96,15 @@ class CoseKey(dict):
         return CoseKey.for_alg(alg)(cose)
 
     @staticmethod
-    def supported_algorithms():
+    def supported_algorithms() -> Sequence[int]:
         """Get a list of all supported algorithm identifiers"""
-        if ed25519:
-            algs = (ES256, EdDSA, PS256, RS256)
-        else:
-            algs = (ES256, PS256, RS256)
+        algs: Sequence[Type[CoseKey]] = [ES256, EdDSA, ES384, ES512, PS256, RS256]
         return [cls.ALGORITHM for cls in algs]
 
 
+T_CoseKey = TypeVar("T_CoseKey", bound=CoseKey)
+
+
 class UnsupportedKey(CoseKey):
     """A COSE key with an unsupported algorithm."""
 
@@ -149,6 +145,60 @@ class ES256(CoseKey):
         return cls({1: 2, 3: cls.ALGORITHM, -1: 1, -2: data[1:33], -3: data[33:65]})
 
 
+class ES384(CoseKey):
+    ALGORITHM = -35
+    _HASH_ALG = hashes.SHA384()
+
+    def verify(self, message, signature):
+        if self[-1] != 1:
+            raise ValueError("Unsupported elliptic curve")
+        ec.EllipticCurvePublicNumbers(
+            bytes2int(self[-2]), bytes2int(self[-3]), ec.SECP384R1()
+        ).public_key(default_backend()).verify(
+            signature, message, ec.ECDSA(self._HASH_ALG)
+        )
+
+    @classmethod
+    def from_cryptography_key(cls, public_key):
+        pn = public_key.public_numbers()
+        return cls(
+            {
+                1: 2,
+                3: cls.ALGORITHM,
+                -1: 1,
+                -2: int2bytes(pn.x, 48),
+                -3: int2bytes(pn.y, 48),
+            }
+        )
+
+
+class ES512(CoseKey):
+    ALGORITHM = -36
+    _HASH_ALG = hashes.SHA512()
+
+    def verify(self, message, signature):
+        if self[-1] != 1:
+            raise ValueError("Unsupported elliptic curve")
+        ec.EllipticCurvePublicNumbers(
+            bytes2int(self[-2]), bytes2int(self[-3]), ec.SECP521R1()
+        ).public_key(default_backend()).verify(
+            signature, message, ec.ECDSA(self._HASH_ALG)
+        )
+
+    @classmethod
+    def from_cryptography_key(cls, public_key):
+        pn = public_key.public_numbers()
+        return cls(
+            {
+                1: 2,
+                3: cls.ALGORITHM,
+                -1: 1,
+                -2: int2bytes(pn.x, 64),
+                -3: int2bytes(pn.y, 64),
+            }
+        )
+
+
 class RS256(CoseKey):
     ALGORITHM = -257
     _HASH_ALG = hashes.SHA256()
@@ -210,7 +260,7 @@ class EdDSA(CoseKey):
 
 class RS1(CoseKey):
     ALGORITHM = -65535
-    _HASH_ALG = hashes.SHA1()
+    _HASH_ALG = hashes.SHA1()  # nosec
 
     def verify(self, message, signature):
         rsa.RSAPublicNumbers(bytes2int(self[-2]), bytes2int(self[-1])).public_key(
