@@ -25,24 +25,25 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
-from __future__ import absolute_import, unicode_literals
+from __future__ import annotations
 
 from .hid import CTAPHID
-from .utils import websafe_encode, websafe_decode, bytes2int
+from .ctap import CtapDevice
+from .utils import websafe_encode, websafe_decode, bytes2int, ByteBuffer
 from .cose import ES256
 from .attestation import FidoU2FAttestation
 from enum import IntEnum, unique
-from binascii import b2a_hex
+from dataclasses import dataclass
 import struct
-import six
 
 
 @unique
 class APDU(IntEnum):
     """APDU response codes."""
+
     OK = 0x9000
     USE_NOT_SATISFIED = 0x6985
-    WRONG_DATA = 0x6a80
+    WRONG_DATA = 0x6A80
 
 
 class ApduError(Exception):
@@ -53,15 +54,16 @@ class ApduError(Exception):
     :param data: APDU response body.
 
     """
-    def __init__(self, code, data=b''):
+
+    def __init__(self, code: int, data: bytes = b""):
         self.code = code
         self.data = data
 
     def __repr__(self):
-        return 'APDU error: 0x{:04X} {:d} bytes of data'.format(
-            self.code, len(self.data))
+        return f"APDU error: 0x{self.code:04X} {len(self.data):d} bytes of data"
 
 
+@dataclass(init=False)
 class RegistrationData(bytes):
     """Binary response data for a CTAP1 registration.
 
@@ -72,32 +74,38 @@ class RegistrationData(bytes):
         encoded.
     :ivar signature: Attestation signature.
     """
+
+    public_key: bytes
+    key_handle: bytes
+    certificate: bytes
+    signature: bytes
+
     def __init__(self, _):
-        super(RegistrationData, self).__init__()
+        super().__init__()
 
-        if six.indexbytes(self, 0) != 0x05:
-            raise ValueError('Reserved byte != 0x05')
+        reader = ByteBuffer(self)
+        if reader.unpack("B") != 0x05:
+            raise ValueError("Reserved byte != 0x05")
 
-        self.public_key = self[1:66]
-        kh_len = six.indexbytes(self, 66)
-        self.key_handle = self[67:67+kh_len]
+        self.public_key = reader.read(65)
+        self.key_handle = reader.read(reader.unpack("B"))
 
-        cert_offs = 67 + kh_len
-        cert_len = six.indexbytes(self, cert_offs + 1)
-        if cert_len > 0x80:
+        cert_buf = reader.read(2)  # Tag and first length byte
+        cert_len = cert_buf[1]
+        if cert_len > 0x80:  # Multi-byte length
             n_bytes = cert_len - 0x80
-            cert_len = bytes2int(self[cert_offs+2:cert_offs+2+n_bytes]) \
-                + n_bytes
-        cert_len += 2
-        self.certificate = self[cert_offs:cert_offs+cert_len]
-        self.signature = self[cert_offs+cert_len:]
+            len_bytes = reader.read(n_bytes)
+            cert_buf += len_bytes
+            cert_len = bytes2int(len_bytes)
+        self.certificate = cert_buf + reader.read(cert_len)
+        self.signature = reader.read()
 
     @property
-    def b64(self):
+    def b64(self) -> str:
         """Websafe base64 encoded string of the RegistrationData."""
         return websafe_encode(self)
 
-    def verify(self, app_param, client_param):
+    def verify(self, app_param: bytes, client_param: bytes) -> None:
         """Verify the included signature with regard to the given app and client
         params.
 
@@ -105,25 +113,16 @@ class RegistrationData(bytes):
         :param client_param: SHA256 hash of the ClientData used for the request.
         """
         FidoU2FAttestation.verify_signature(
-            app_param, client_param, self.key_handle, self.public_key,
-            self.certificate, self.signature)
-
-    def __repr__(self):
-        return ("RegistrationData(public_key: h'%s', key_handle: h'%s', "
-                "certificate: h'%s', signature: h'%s')") % tuple(
-                    b2a_hex(x).decode() for x in (
-                        self.public_key,
-                        self.key_handle,
-                        self.certificate,
-                        self.signature
-                    )
-                )
-
-    def __str__(self):
-        return '%r' % self
+            app_param,
+            client_param,
+            self.key_handle,
+            self.public_key,
+            self.certificate,
+            self.signature,
+        )
 
     @classmethod
-    def from_b64(cls, data):
+    def from_b64(cls, data: str) -> RegistrationData:
         """Parse a RegistrationData from a websafe base64 encoded string.
 
         :param data: Websafe base64 encoded string.
@@ -132,6 +131,7 @@ class RegistrationData(bytes):
         return cls(websafe_decode(data))
 
 
+@dataclass(init=False)
 class SignatureData(bytes):
     """Binary response data for a CTAP1 authentication.
 
@@ -140,19 +140,25 @@ class SignatureData(bytes):
     :ivar counter: Signature counter.
     :ivar signature: Cryptographic signature.
     """
-    def __init__(self, _):
-        super(SignatureData, self).__init__()
 
-        self.user_presence = six.indexbytes(self, 0)
-        self.counter = struct.unpack('>I', self[1:5])[0]
-        self.signature = self[5:]
+    user_presence: int
+    counter: int
+    signature: bytes
+
+    def __init__(self, _):
+        super().__init__()
+
+        reader = ByteBuffer(self)
+        self.user_presence = reader.unpack("B")
+        self.counter = reader.unpack(">I")
+        self.signature = reader.read()
 
     @property
-    def b64(self):
+    def b64(self) -> str:
         """str: Websafe base64 encoded string of the SignatureData."""
         return websafe_encode(self)
 
-    def verify(self, app_param, client_param, public_key):
+    def verify(self, app_param: bytes, client_param: bytes, public_key: bytes) -> None:
         """Verify the included signature with regard to the given app and client
         params, using the given public key.
 
@@ -163,16 +169,8 @@ class SignatureData(bytes):
         m = app_param + self[:5] + client_param
         ES256.from_ctap1(public_key).verify(m, self.signature)
 
-    def __repr__(self):
-        return ('SignatureData(user_presence: 0x%02x, counter: %d, '
-                "signature: h'%s'") % (self.user_presence, self.counter,
-                                       b2a_hex(self.signature))
-
-    def __str__(self):
-        return '%r' % self
-
     @classmethod
-    def from_b64(cls, data):
+    def from_b64(cls, data: str) -> SignatureData:
         """Parse a SignatureData from a websafe base64 encoded string.
 
         :param data: Websafe base64 encoded string.
@@ -181,22 +179,24 @@ class SignatureData(bytes):
         return cls(websafe_decode(data))
 
 
-class CTAP1(object):
+class Ctap1:
     """Implementation of the CTAP1 specification.
 
     :param device: A CtapHidDevice handle supporting CTAP1.
     """
+
     @unique
     class INS(IntEnum):
         REGISTER = 0x01
         AUTHENTICATE = 0x02
         VERSION = 0x03
 
-    def __init__(self, device):
-
+    def __init__(self, device: CtapDevice):
         self.device = device
 
-    def send_apdu(self, cla=0, ins=0, p1=0, p2=0, data=b''):
+    def send_apdu(
+        self, cla: int = 0, ins: int = 0, p1: int = 0, p2: int = 0, data: bytes = b""
+    ) -> bytes:
         """Packs and sends an APDU for use in CTAP1 commands.
         This is a low-level method mainly used internally. Avoid calling it
         directly if possible, and use the get_version, register, and
@@ -210,28 +210,24 @@ class CTAP1(object):
         :return: The response APDU data of a successful request.
         :raise: ApduError
         """
-        size = len(data)
-        size_h = size >> 16 & 0xff
-        size_l = size & 0xffff
-        apdu = struct.pack('>BBBBBH', cla, ins, p1, p2, size_h, size_l) \
-            + data + b'\0\0'
+        apdu = struct.pack(">BBBBBH", cla, ins, p1, p2, 0, len(data)) + data + b"\0\0"
 
         response = self.device.call(CTAPHID.MSG, apdu)
-        status = struct.unpack('>H', response[-2:])[0]
+        status = struct.unpack(">H", response[-2:])[0]
         data = response[:-2]
         if status != APDU.OK:
             raise ApduError(status, data)
         return data
 
-    def get_version(self):
+    def get_version(self) -> str:
         """Get the U2F version implemented by the authenticator.
         The only version specified is "U2F_V2".
 
         :return: A U2F version string.
         """
-        return self.send_apdu(ins=CTAP1.INS.VERSION).decode()
+        return self.send_apdu(ins=Ctap1.INS.VERSION).decode()
 
-    def register(self, client_param, app_param):
+    def register(self, client_param: bytes, app_param: bytes) -> RegistrationData:
         """Register a new U2F credential.
 
         :param client_param: SHA256 hash of the ClientData used for the request.
@@ -239,11 +235,16 @@ class CTAP1(object):
         :return: The registration response from the authenticator.
         """
         data = client_param + app_param
-        response = self.send_apdu(ins=CTAP1.INS.REGISTER, data=data)
+        response = self.send_apdu(ins=Ctap1.INS.REGISTER, data=data)
         return RegistrationData(response)
 
-    def authenticate(self, client_param, app_param, key_handle,
-                     check_only=False):
+    def authenticate(
+        self,
+        client_param: bytes,
+        app_param: bytes,
+        key_handle: bytes,
+        check_only: bool = False,
+    ) -> SignatureData:
         """Authenticate a previously registered credential.
 
         :param client_param: SHA256 hash of the ClientData used for the request.
@@ -253,8 +254,9 @@ class CTAP1(object):
             determine if a key handle is known.
         :return: The authentication response from the authenticator.
         """
-        data = client_param + app_param \
-            + struct.pack('>B', len(key_handle)) + key_handle
+        data = (
+            client_param + app_param + struct.pack(">B", len(key_handle)) + key_handle
+        )
         p1 = 0x07 if check_only else 0x03
-        response = self.send_apdu(ins=CTAP1.INS.AUTHENTICATE, p1=p1, data=data)
+        response = self.send_apdu(ins=Ctap1.INS.AUTHENTICATE, p1=p1, data=data)
         return SignatureData(response)
